<?php

namespace sellvia\Media\Upload;

use sellvia\Media\Media;

abstract class uploadExternalMedia {

    protected $options = [];

    public function __construct(){

        $this->options = sellvia_get_images_settings();
    }

    public function actions( $post ) {

        if( isset( $post['sellvia_action'] ) && current_user_can( 'activate_plugins' ) ) {

            $action = 'action_' . $post['sellvia_action'];
            $args        = isset( $post['args'] ) ? $post['args'] : $post;
            $data        = [];

            if( is_array( $args ) )
                $data = $args;
            else
                parse_str( $args, $data );


            if( method_exists( $this, $action ) ) {

                return $this->$action( $data );
            }
        }

        return [ 'error' => __( 'Undefined action', 'sel' ) ];
    }

    public function action_info(){

        $current = intval( get_option('sellvia_current_upload_external_media_product', 0) );

        $total = (int) $this->count_products();

        if( $current >= $total ){
            $current = 0;
            update_option( 'sellvia_current_upload_external_media_product', 0 ,false);
        }

        return array(
            'result' => true,
            'total'   => $total,
            'current' => $current,
            'settings' => $this->options,
        );
    }

    protected function action_save_settings( $args ){

        if( ! isset( $args['settings'] ) || !is_array( $args['settings'] ) ) {
            return [
                'result'  => false,
                'message' => __('Wrong data', 'sel')
            ];
        }

        $income = sellvia_prepare_images_settings( $args['settings'] );

        update_option('sellvia-images-settings', $income );

        return [
            'result'  => true,
            'message' => __('Images settings have been updated', 'sel')
        ];
    }

    protected function action_first_product( $args ) {

        if( ! isset( $args['settings'] ) || !is_array( $args['settings'] ) ) {
            return [
                'result'  => false,
                'message' => __('Wrong data', 'sel')
            ];
        }

        $income = sellvia_prepare_images_settings( $args['settings'] );

        update_option('sellvia-images-settings', $income );

        $this->options = sellvia_get_images_settings();

        update_option( 'sellvia_current_upload_external_media_product', 0 ,false);

        return $this->action_next_product();
    }

    protected function action_next_product() {

        $current = intval( get_option( 'sellvia_current_upload_external_media_product', 0 ) );
        $total   = (int)$this->count_products();
        $row     = $current < $total ? $this->get_next_product( $current ) : false;

        $result = [
            'result'  => true,
            'total'   => $total,
            'row'     => $row,
            'current' => $current,
        ];

        $current = intval( $current ) + 1;

        update_option( 'sellvia_current_upload_external_media_product', $current, false );

        if( !empty( $row ) ) {

            $images = $this->list_external_images_post( $row['post_id'] );

            $result['images'] = $images;

        }else{
            $result['type']    = 'success';
            $result['title']   = __( 'Success', 'sel');
            $result['message'] = __( 'Images have been uploaded', 'sel');
        }
        return $result;
    }

    protected function action_upload_images( $args ){

        if( ! isset( $args['post_id'] ) ) {
            return[
                'result' => false,
                'type' => 'error',
                'title' => __('Error', 'sel'),
                'message' =>  __( 'Wrong "post_id" parameter', 'sel' )
            ];
        };


        $result = $this->upload_images( $args['post_id'], $args['images'] ?? [] );

        if( !$result['success']) {
            return [
                'result' => $result['success'],
                'type' => 'error',
                'title' => __('Error', 'sel'),
                'message' => isset($result['message']) ? $result['message'] : __('Something went wrong', 'sel'),
            ];
        }

        return[
            'result' => $result['success'],
            'data' => $result['data'],
        ];

    }

    protected function upload_images( $post_id, $images ){

        $media = new Media();

        $uploaded = false;
        $foo = [];

        if( !empty($images) ) {

            foreach ( $images as $image ){

                if ( $uploaded || intval( $image['local'] ) === 1 ) {
                    $foo[] = $image;
                    continue;
                }

                if ( intval( $image['local'] )  === 0 ) {

                    $media->localize_attachment( $image['post_id'] );

                    if ( $image['place'] == 'content' ) {

                        $media->replaceImgContent( $post_id, [
                            'post_id' => $image['post_id'],
                            'url'     => $image['url'],
                        ]);
                    }

                    if ( $image['place'] == 'short' ) {

                        $media->replaceImgShort( $post_id, [
                            'post_id' => $image['post_id'],
                            'url'     => $image['url'],
                        ]);
                    }

                    $image['local'] = 1;

                    $foo[] = $image;
                    $uploaded = true;
                }

            }
        }

        return [
            'success' => true,
            'data' => [
                'post_id' => $post_id,
                'images' => $foo
            ]
        ];

    }

    protected function list_external_images_post( $post_id ){

        $post_id = intval($post_id);

        $images = [];

        if( $post_id == 0 )
            return $images;

        if ($this->options['feature_images'] === 1) {

            $images = array_merge( $images, $this->link_feature( $post_id ) );
        }

        if ($this->options['gallery_images'] === 1) {

            $images = array_merge( $images, $this->link_gallery( $post_id ) );
        }

        if ($this->options['variation_images'] === 1) {

            $images = array_merge( $images, $this->link_variation( $post_id ) );
        }

        if ($this->options['content_images'] === 1) {

            $images = array_merge( $images, $this->link_content( $post_id ) );
            $images = array_merge( $images, $this->link_short( $post_id ) );
        }

        if ($this->options['reviews_images'] === 1) {

            $images = array_merge( $images, $this->link_reviews( $post_id ) );
        }

        return $images;
    }

    protected function count_products(){

        global $wpdb;

        $var = $wpdb->get_var(
            "SELECT COUNT(posts.ID) 
                FROM {$wpdb->posts} posts
                    INNER JOIN {$wpdb->postmeta} m ON m.post_id = posts.ID
                WHERE m.meta_key = 'sellvia_post_id' 
                    AND posts.post_type = 'product' 
                    AND posts.post_status IN('publish','draft','importlist')"
        );

        return ! empty( $var ) ? $var : 0;
    }

    protected function link_content( $post_id ) {

        $post    = get_post( $post_id );
        $content = $post->post_content;

        $media = new Media();

        $images  = $media->searchImgInContent( $content );

        $foo = [];

        if( $images ) foreach( $images as $url ) {

            if( $media->is_url_img( $url ) && $media->isExternal( $url ) ) {

                $image_id = $media->create_external_attachment( $post_id, $url );

                if( !$image_id ) {
                    continue;
                }

                $foo[] = [
                    'place'   => 'content',
                    'url'     => $url,
                    'post_id' => intval( $image_id ),
                    'local'   => 0
                ];
            }
        }
        return $foo;
    }

    protected function link_short( $post_id ){

        $post    = get_post( $post_id );
        $content = $post->post_excerpt;

        $media = new Media();

        $images  = $media->searchImgInContent( $content );

        $foo = [];

        if( $images ) foreach( $images as $url ) {

            if( $media->is_url_img( $url ) && $media->isExternal( $url ) ) {

                $image_id = $media->create_external_attachment( $post_id, $url );

                if( !$image_id ) {
                    continue;
                }

                $foo[] = [
                    'place'   => 'short',
                    'url'     => $url,
                    'post_id' => intval( $image_id ),
                    'local'   => 0
                ];
            }
        }
        return $foo;

    }

    protected function link_reviews( $post_id ){

        $foo = [];

        global $wpdb;

        $comments = $wpdb->get_results(
            $wpdb->prepare( "SELECT c.comment_ID, cm.meta_value as images FROM {$wpdb->comments} c 
                INNER JOIN {$wpdb->commentmeta} cm ON cm.comment_id = c.comment_ID
                WHERE c.comment_post_ID = %d AND cm.meta_key = 'images' ORDER BY c.comment_ID", $post_id), ARRAY_A );

        if( empty( $comments ) ){
            return [];
        }

        $media = new Media();

        foreach ( $comments as $comment ){

            if( empty( $comment['images'] ) ){
                continue;
            }

            $imgs = maybe_unserialize( $comment['images'] );

            if( empty( $imgs ) ){
                continue;
            }

            foreach ( $imgs as $k => $img ){

                if( $media->is_url_img( $img ) && $media->isExternal( $img ) ){

                    $image_id = $media->create_external_attachment( $post_id, $img );

                    if( !$image_id ) {
                        continue;
                    }

                    $foo[] = [
                        'place'      => 'review',
                        'url'        => $img,
                        'post_id'    => intval( $image_id ),
                        'local'      => 0
                    ];

                    $imgs[$k] = $image_id;

                } elseif ( is_numeric( $img ) ){

                    if( !$media->is_external_attachment( $img ) ){
                        continue;
                    }

                    $url = get_post_meta( $img, '_wp_attached_file', true);

                    if( ! $media->is_url_img( $url ) || ! $media->isExternal( $url ) ){
                        continue;
                    }

                    $foo[] = [
                        'place'      => 'review',
                        'url'        => $url,
                        'post_id'    => intval( $img ),
                        'local'      => 0
                    ];
                }

                update_comment_meta( $comment['comment_ID'], 'images',  $imgs );
            }
        }
        return $foo;
    }
}
