<?php


namespace sellvia\ProductUpdate;


abstract class ProductUpdate {

    public $settings = [
        'status'         => 'draft',
        'notify_status'  => 0,
        'variant'        => 'disable',
        'notify_variant' => 0,
        'cost'           => 'update',
        'notify_cost'    => 0,
        'stock'          => 'update',
        'notify_stock'   => 0,
    ];

    public static $default_setting = [
        'status'         => 'draft',
        'notify_status'  => 0,
        'variant'        => 'disable',
        'notify_variant' => 0,
        'cost'           => 'update',
        'notify_cost'    => 0,
        'stock'          => 'update',
        'notify_stock'   => 0,
    ];

    public $price_model = [];

    public function __construct(){

        $this->initSettings();

        $this->initFilters();
    }

    protected function initSettings(){

        $this->settings = sellvia_parse_args( self::$default_setting,  get_option( 'sellvia_update_product_settings', [] ) );

        $this->price_model = sellvia_get_price_model_settings();
    }

    protected function initFilters(){

        add_filter( 'sellvia_import_price', function( $price ) {

            if( floatval( $this->price_model['value'] ) === 100 ) {
                return $price;
            }

            return sellvia_floatvalue( ( floatval( $this->price_model['value'] ) / 100 ) * floatval( $price ) );
        } );
    }

    public function actions( $post ) {

        if( isset( $post['sellvia_action'] ) && current_user_can( 'activate_plugins' ) ) {

            $action = 'action_' . $post['sellvia_action'];
            $args        = isset( $post['args'] ) ? $post['args'] : $post;
            $data        = [];

            if( is_array( $args ) )
                $data = $args;
            else
                parse_str( $args, $data );


            if( method_exists( $this, $action ) ) {

                return $this->$action( $data );
            }
        }

        return [ 'error' => __( 'Undefined action', 'sel' ) ];
    }

    public function action_save_settings( $args ){

        $args['settings'] = isset( $args['settings'] ) ? $args['settings'] : [];

        $args['settings']['notify_status']  = isset($args['settings']['notify_status'])  ? intval( $args['settings']['notify_status'] )  : 0;
        $args['settings']['notify_variant'] = isset($args['settings']['notify_variant']) ? intval( $args['settings']['notify_variant'] ) : 0;
        $args['settings']['notify_cost']    = isset($args['settings']['notify_cost'])    ? intval( $args['settings']['notify_cost'] )    : 0;
        $args['settings']['notify_stock']   = isset($args['settings']['notify_stock'])   ? intval( $args['settings']['notify_stock'] )   : 0;

        $args['settings']['status']         = $args['settings']['status']  ?? 'draft';
        $args['settings']['variant']        = $args['settings']['variant'] ?? 'disable';

        update_option( 'sellvia_update_product_settings' , sellvia_parse_args( self::$default_setting, $args['settings'] ), false );

        return [
            'result' => true,
            'type' => 'success',
            'title' => __('Success', 'sel'),
            'message' => __('Settings have been saved'),
            'settings' => sellvia_parse_args( self::$default_setting,  get_option( 'sellvia_update_product_settings', [] ) ),
        ];

    }

    public function action_info(){

        $current = intval( get_option( 'sellvia_current_update_product', 0 ) );

        $total = (int) $this->count_products();

        if( $current >= $total ){
            $current = 0;
            update_option( 'sellvia_current_update_product', 0 ,false);
        }

        return array(
            'result' => true,
            'total'   => $total,
            'current' => $current,
            'settings' => $this->settings,
            'values' => [
                'status' => [
                    [
                        'value' => 'doNothing',
                        'label' => __( 'Do nothing', 'sel' ),
                    ],
                    [
                        'value' => 'zero',
                        'label' => __( 'Set quantity to zero', 'sel' ),
                    ],
                    [
                        'value' => 'draft',
                        'label' => __( 'Send to draft/restore', 'sel' ),
                    ],
                ],
                'variant' => [
                    [
                        'value' => 'doNothing',
                        'label' => __( 'Do nothing', 'sel' ),
                    ],
                    [
                        'value' => 'zero',
                        'label' => __( 'Set quantity to zero', 'sel' ),
                    ],
                    [
                        'value' => 'disable',
                        'label' => __( 'Disable/Enable', 'sel' ),
                    ],
                ],
                'cost' => [
                    [
                        'value' => 'doNothing',
                        'label' => __( 'Do nothing', 'sel' ),
                    ],
                    [
                        'value' => 'update',
                        'label' => __( 'Update automatically', 'sel' ),
                    ]
                ],
                'stock' => [
                    [
                        'value' => 'doNothing',
                        'label' => __( 'Do nothing', 'sel' ),
                    ],
                    [
                        'value' => 'update',
                        'label' => __( 'Update automatically', 'sel' ),
                    ]
                ]
            ],
            'labels' => [
                'status'  => __( 'When a product disappears or is out of stock', 'sel' ),
                'variant' => sprintf( __( 'When a product %d disappears or is out of stock', 'sel' ), in_array( SELLVIA_PLUGIN, ['ADSW', 'WOO'] ) ? __( 'variation', 'sel' ) : __( 'variant', 'sel' ) ),
                'cost'    => __( 'Update prices', 'sel' ),//__( 'When the price changes', 'sel' ),
                'stock'   => __( 'Update stock', 'sel' ),//__( 'When stock changes', 'sel' ),
            ],
            'help' => [
                'status'  => __( 'Select an option when a product is no longer available on Sellvia or is out of stock.', 'sel' ),
                'variant' => sprintf( __( 'Select an option when a product %s is no longer available on Sellvia or is out of stock.', 'sel' ), in_array( SELLVIA_PLUGIN, ['ADSW', 'WOO'] ) ? __( 'variation', 'sel' ) : __( 'variant', 'sel' ) ),
                'cost'    => __( 'Select an option when the price of a product on Sellvia changes.', 'sel' ),
                'stock'   => __( 'Select an option when the stock of a product on Sellvia changes.', 'sel' )
            ],
        );
    }

    public function action_first_product( $args ) {

        $args['settings'] = isset( $args['settings'] ) ? $args['settings'] : [];

        $args['settings']['notify_status']  = isset($args['settings']['notify_status'])  ? intval( $args['settings']['notify_status'] )  : 0;
        $args['settings']['notify_variant'] = isset($args['settings']['notify_variant']) ? intval( $args['settings']['notify_variant'] ) : 0;
        $args['settings']['notify_cost']    = isset($args['settings']['notify_cost'])    ? intval( $args['settings']['notify_cost'] )    : 0;
        $args['settings']['notify_stock']   = isset($args['settings']['notify_stock'])   ? intval( $args['settings']['notify_stock'] )   : 0;

        $args['settings']['status']         = $args['settings']['status']  ?? 'draft';
        $args['settings']['variant']        = $args['settings']['variant'] ?? 'disable';

        update_option( 'sellvia_update_product_settings' , sellvia_parse_args( self::$default_setting, $args['settings'] ), false );

        update_option( 'sellvia_current_update_product', 0 ,false);

        return $this->action_next_product();
    }

    public function action_next_product(){

        $current = intval( get_option( 'sellvia_current_update_product', 0 ) );
        $total   = (int)$this->count_products();
        $row     = $current < $total ? $this->get_next_product( $current ) : false;

        $result = [
            'result'  => true,
            'total'   => $total,
            'row'     => $row,
            'current' => $current,
        ];

        $current = intval( $current ) + 1;
        update_option( 'sellvia_current_update_product', $current, false );
        update_option( 'sellvia_last_update_products', current_time( 'timestamp' ) );

        if( !empty( $row ) ) {

            $response = $this->update( $row['post_id'], $row['sellvia_post_id'] );

            if( isset( $response['error'] ) ){

                $result['result']                = false;
                $result['type']                  = 'error';
                $result['title']                 = __( 'Error', 'sel');
                $result['message']               = $response['error'];
                $result['row']['message']        = $response['error'];
                $result['row']['message_status'] = 'error';
                $result['row']['html_price']     = '';
                $result['row']['changePrice']    = false;
                $result['row']['changeQuantity'] = false;
                $result['row']['disappeared']    = false;
            } else {

                $result['row']['message']        = $response['message'];
                $result['row']['message_status'] = $response['result'] ? 'success' : 'warning';
                $result['row']['html_price']     = $this->preparePricing();
                $result['row']['html_quantity']  = $this->prepareQuantity();
                $result['row']['changePrice']    = $this->getChangePrice();
                $result['row']['changeQuantity'] = $this->getChangeQuantity();
                $result['row']['disappeared']    = $this->getDisappeared();
            }

        }else{
            $result['type']    = 'success';
            $result['title']   = __( 'Success', 'sel');
            $result['message'] = __( 'Update product has been successfully finished', 'sel');
        }
        return $result;
    }

    protected function action_get_product_update_settings( $args ){

        if( ! isset( $args['post_id'] ) ) {
            return[
                'result' => false,
                'type' => 'error',
                'title' => __('Error', 'sel'),
                'message' =>  __( 'Wrong "post_id"', 'sel' )
            ];
        }

        $sellvia_post_id = get_post_meta( $args['post_id'], 'sellvia_post_id', true );
        $sellvia_need_update = get_post_meta( $args['post_id'], 'sellvia_need_update', true );

        if( $sellvia_need_update === false ){
            return[
                'result' => false,
                'type' => 'error',
                'title' => __('Error', 'sel'),
                'message' =>  __( 'Product not found', 'sel' )
            ];
        }

        $sellvia_need_update = intval( $sellvia_need_update );
        return [
            'result'              => true,
            'sellvia_post_id'     => $sellvia_post_id,
            'sellvia_need_update' => $sellvia_need_update,
            'settings'            => $this->settings
        ];
    }

    protected function action_save_product_update_settings( $args ){

        if( ! isset( $args['post_id'] ) ) {
            return[
                'result' => false,
                'type' => 'error',
                'title' => __('Error', 'sel'),
                'message' =>  __( 'Wrong "post_id"', 'sel' )
            ];
        }

        if( ! isset( $args['sellvia_need_update'] ) ) {
            return[
                'result' => false,
                'type' => 'error',
                'title' => __('Error', 'sel'),
                'message' =>  __( 'Wrong params', 'sel' )
            ];
        }

        update_post_meta( $args['post_id'], 'sellvia_need_update', intval( $args['sellvia_need_update'] ) === 1 ? 1 : 0);


        return [
            'result'  => true,
            'message' =>  __( 'Product update settings have been saved', 'sel' )
        ];
    }

    protected function action_single_update( $args ){

        if( ! isset( $args['post_id'] ) ) {
            return[
                'result' => false,
                'type' => 'error',
                'title' => __('Error', 'sel'),
                'message' =>  __( 'Wrong "post_id"', 'sel' )
            ];
        }

        $args['settings'] = isset( $args['settings'] ) ? $args['settings'] : [];

        $this->settings = sellvia_parse_args( $this->settings, $args['settings'] );

        $sellvia_post_id = get_post_meta( $args['post_id'], 'sellvia_post_id', true );

        $response = $this->update( $args['post_id'], $sellvia_post_id );

        if( isset( $response['error'] ) ){
            return [
                'result' => false,
                'type' => 'error',
                'title' => __( 'Error', 'sel'),
                'message' => $response['error']
            ];
        } else {
            return [
                'result' => true,
                'type' => $response['result'] ? 'success' : 'warning',
                'title' => $response['result'] ? __( 'Success', 'sel') : __( 'Warning', 'sel'),
                'message' => $response['message']
            ];
        }

    }

    protected function count_products(){

        global $wpdb;

        $var = $wpdb->get_var(
            "SELECT COUNT(posts.ID) FROM {$wpdb->posts} posts
                INNER JOIN {$wpdb->postmeta} m ON m.post_id = posts.ID
                INNER JOIN {$wpdb->postmeta} m2 ON m2.post_id = posts.ID
            WHERE m.meta_key = 'sellvia_post_id' 
                AND m2.meta_key = 'sellvia_need_update' 
                AND m2.meta_value = '1' 
                AND posts.post_type = 'product' 
                AND posts.post_status IN('publish','draft','importlist')"
        );

        return ! empty( $var ) ? $var : 0;
    }

    static function call( $path, $args = [] ) {

        $keys = sellvia_get_api_keys();

        $public_key  = $keys['public_key'];
        $secret_key  = $keys['secret_key'];

        $auth_args = [
            'public_key'  => $public_key,
            'signature'   => md5( $public_key . $secret_key ),
            'domain'      => get_home_url(),
            'plugin'      => 'sellvia',
        ];

        $args = array_merge( $args, $auth_args );

//        var_dump( json_encode($args) );

        $response = wp_remote_post( SELLVIA_REQUEST_PATH . $path, [
            'method'    => 'POST',
            'timeout'   => 120,
            'sslverify' => false,
            'headers'   => [],
            'body'      => json_encode( $args )
        ] );

        if( is_wp_error( $response ) )
            return [ 'error' => $response->get_error_message() ];

        if( ! wp_remote_retrieve_response_code( $response ) == '200' )
            return [ 'error' => __( 'The response from the server has not been received', 'sel' ) ];

        return json_decode( wp_remote_retrieve_body( $response ), true );
    }

    public static function preparePrices( $prices ) {

        if( !empty( $prices ) ) {
            foreach ( $prices as &$price ) {
                $price = apply_filters('sellvia_import_price', $price);
            }
        }

        return $prices;
    }

    public static function prepareSkuAttr( $skuAttr ) {

        $skuAttr = maybe_unserialize( $skuAttr );

        if( !empty( $skuAttr ) ) {
            foreach ( $skuAttr as &$item ) {
                $item['price'] = apply_filters('sellvia_import_price', $item['price'] );
                $item['salePrice'] = apply_filters('sellvia_import_price',  $item['salePrice'] );
            }
        }

        return maybe_serialize( $skuAttr );
    }
}